<?php

declare(strict_types=1);

namespace ProxyManager\Proxy\Exception;

use RuntimeException;

/**
 * Remote object exception
 *
 * @deprecated this exception is not in use anymore, and should not be relied upon
 */
class RemoteObjectException extends RuntimeException
{
}
