<?php

declare(strict_types=1);

namespace ProxyManagerTestAsset;

/**
 * Class with one abstract public method
 *
 * @author Marco Pivetta <ocramius@gmail.com>
 * @license MIT
 */
abstract class ClassWithAbstractPublicMethod
{
    /**
     * @return void
     */
    abstract public function publicAbstractMethod();
}
