<?php

declare(strict_types=1);

namespace ProxyManagerTestAsset;

/**
 * Base test class to play around with public properties
 *
 * @author Marco Pivetta <ocramius@gmail.com>
 * @license MIT
 */
class ClassWithPublicProperties
{
    /** @var string */
    public $property0 = 'property0';

    /** @var string */
    public $property1 = 'property1';

    /** @var string */
    public $property2 = 'property2';

    /** @var string */
    public $property3 = 'property3';

    /** @var string */
    public $property4 = 'property4';

    /** @var string */
    public $property5 = 'property5';

    /** @var string */
    public $property6 = 'property6';

    /** @var string */
    public $property7 = 'property7';

    /** @var string */
    public $property8 = 'property8';

    /** @var string */
    public $property9 = 'property9';
}
