<?php

declare(strict_types=1);

namespace ProxyManagerTestAsset;

/**
 * Class with one protected method
 *
 * @author Marco Pivetta <ocramius@gmail.com>
 * @license MIT
 */
abstract class ClassWithProtectedMethod
{
    protected function protectedMethod() : void
    {
    }
}
