<?php

declare(strict_types=1);

namespace ProxyManagerTestAsset\ProxyGenerator\LazyLoading\MethodGenerator;

/**
 * Test asset with 2 public properties
 *
 * @author Marco Pivetta <ocramius@gmail.com>
 * @license MIT
 */
class ClassWithTwoPublicProperties
{
    /**
     * @var mixed
     */
    public $bar;

    /**
     * @var mixed
     */
    public $baz;
}
