<?php

declare(strict_types=1);

namespace ProxyManagerTestAsset;

/**
 * Base test class to play around with private properties
 *
 * @author Marco Pivetta <ocramius@gmail.com>
 * @license MIT
 */
class ClassWithPrivateProperties
{
    private $property0 = 'property0';

    private $property1 = 'property1';

    private $property2 = 'property2';

    private $property3 = 'property3';

    private $property4 = 'property4';

    private $property5 = 'property5';

    private $property6 = 'property6';

    private $property7 = 'property7';

    private $property8 = 'property8';

    private $property9 = 'property9';
}
